/**
 * Back 2 Browser Bytecode Translator
 * Copyright (C) 2012-2015 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://opensource.org/licenses/GPL-2.0.
 */
package org.apidesign.bck2brwsr.flow;

import java.util.Arrays;
import org.apidesign.vm4brwsr.Bck2Brwsr;

/** Poweful flow analyzer. Based on internals used by Graal JVM
 * compiler.
 *
 * @author Jaroslav Tulach
 */
public final class GraalFlowAnalyzer {
    private GraalFlowAnalyzer() {
    }
    
    public static Bck2Brwsr.Flow.Analyzer getDefault() {
        return Impl.DEFAULT;
    }

    private static class Impl implements Bck2Brwsr.Flow.Analyzer {
        static Impl DEFAULT = new Impl();
        
        private Impl() {
        }

        @Override
        public boolean analyze(final Bck2Brwsr.Flow result) {
            /*
            // Delete the previous line and fix pom.xml, David.
            try {
                new com.oracle.graal.tag.codegen.BCCodeGenerator(
                    com.oracle.graal.tag.BCTagDriver.tagGraph(result.getMethodByteCode())
                ) {
                    @Override
                    protected void emitIfStart(int bci, String txt) {
                        System.err.println("if { // " + bci);
                        result.beginIfAt(bci);
                    }
                    
                    
                    @Override
                    protected void emitEnd(int bci) {
                        System.err.println("} // " + bci);
                        result.endAt(bci);
                    }

                    @Override
                    protected void emitElseHeader(int bci) {
                        System.err.println("else { //" + bci);
                        result.beginElseAt(bci);
                    }

                    @Override
                    protected void emitLoopHeader(int bci) {
                        System.err.println("for (;;) { // " + bci);
                        result.beginLoopAt(bci);
                    }

                    @Override
                    protected void emitBreak(int bci) {
                        System.err.println("break; // " + bci);
                        result.breakAt(bci);
                    }

                    @Override
                    protected void emit(String txt) {
                    }
                    
                }.GenBCCode();
                return true;
            } catch (Exception e) {
                e.printStackTrace();
            }
            // */
            
            byte[] simpleLoopTestMock = { 3, 60, 3, 61, 28, 26, -94, 0, 13, 27, 28, 96, 60, -124, 2, 1, -89, -1, -12, 27, 26, 104, -84 };
            if (Arrays.equals(
                simpleLoopTestMock,
                result.getMethodByteCode()
            )) {
                result.beginLoopAt(4); // for (;;) { // 4
                result.beginIfAt(6); // if { // 6
                result.breakAt(9); // ;break; // 9
                result.endAt(9); // ;} // 9
                result.beginElseAt(9); // else { //9
                result.endAt(19); // } // 19
                result.endAt(19); // } // 19
                return true;
            }
            
            return false;
        }
    }
}
